/*

	mplayerplug-in

	License: GPL

	This file contains User Interface routines for the mplayerplug-in

*/

#include "mplayerplug-in.h"
#include <X11/xpm.h>
#include <X11/cursorfont.h>
#include "logo.xpm"

#define BORDER               10
#define LOGO_WIDTH          348
#define LOGO_HEIGHT          48

#ifndef HAVE_X11_XPM
#error libXpm has not been found. Compilation cannot continue
#endif

static XFontStruct *font = NULL;
static Pixmap logo, logomask;

void Initialize(Display * dpy, Drawable d, GC gc)
{
    XpmCreatePixmapFromData(dpy, d, logo_xpm, &logo, &logomask, NULL);
    font = XQueryFont(dpy, XGContextFromGC(gc));
}

void DrawLogo(Display * dpy, Drawable d, GC gc)
{
    XGCValues values;

    values.clip_mask = logomask;
    values.clip_x_origin = 0;
    values.clip_y_origin = 0;
    XChangeGC(dpy, gc, GCClipMask | GCClipXOrigin | GCClipYOrigin,
	      &values);
    XCopyArea(dpy, logo, d, gc, 0, 0, LOGO_WIDTH, LOGO_HEIGHT, 0, 0);
    values.clip_mask = (int) NULL;
    XChangeGC(dpy, gc, GCClipMask | GCClipXOrigin | GCClipYOrigin,
	      &values);
}

void DrawProgress(Display * dpy, Window w, int x, int y, int width,
		  int height, int percent)
{
    XGCValues values;
    GC border_gc, fill_gc, progress_gc;
    int pixelpercent;
    XColor Xcolor;

    values.foreground = BlackPixel(dpy, DefaultScreen(dpy));
    border_gc = XCreateGC(dpy, w, GCForeground, &values);
    values.foreground = WhitePixel(dpy, DefaultScreen(dpy));
    fill_gc = XCreateGC(dpy, w, GCForeground, &values);
    XParseColor(dpy, DefaultColormap(dpy, DefaultScreen(dpy)), "blue",
		&Xcolor);
    XAllocColor(dpy, DefaultColormap(dpy, DefaultScreen(dpy)), &Xcolor);
    values.foreground = Xcolor.pixel;
    progress_gc = XCreateGC(dpy, w, GCForeground, &values);
    XFillRectangle(dpy, w, border_gc, x, y, width, height);
    XFillRectangle(dpy, w, fill_gc, x + 1, y + 1, width - 2, height - 2);
    pixelpercent = ((width - 2) - (x + 1)) * (percent / 100.);
    XFillRectangle(dpy, w, progress_gc, x + 1, y + 1, pixelpercent,
		   height - 2);
    XFreeGC(dpy, border_gc);
    XFreeGC(dpy, fill_gc);
    XFreeGC(dpy, progress_gc);
}



void DrawUI(Widget w, NPP instance, char *message, int FullRedraw,
	    int percent)
{

    GC black_gc, white_gc;
    XGCValues values;
    PluginInstance *This;
    const char *id = "mplayerplug-in v" PACKAGE_VERSION;
    int id_width;
    int top, font_height, loops;
    int term, chop;
    Cursor guicursor;
    XSetWindowAttributes attrs;

    This = (PluginInstance *) instance->pdata;
    if (0) {
	printf("Widget: %i\n", w);
	printf("Message: %s\n", message);
    }

    if (w == NULL)
	return;
    if (This->state < STATE_READY)
	return;
    if (message == NULL)
	return;
    if (DESTROYED == 1)
	return;
    if (This->display == NULL)
	return;

    sprintf(This->lastmessage, "%s", message);

    values.foreground =
	BlackPixel((Display *) This->display,
		   DefaultScreen((Display *) This->display));
    black_gc =
	XCreateGC((Display *) This->display, This->window, GCForeground,
		  &values);
    values.foreground =
	WhitePixel((Display *) This->display,
		   DefaultScreen((Display *) This->display));
    white_gc =
	XCreateGC((Display *) This->display, This->window, GCForeground,
		  &values);

    if (font == NULL) {
	FullRedraw = TRUE;
	Initialize((Display *) This->display, This->window, black_gc);
	guicursor = XCreateFontCursor((Display *) This->display, XC_watch);
	//XDefineCursor((Display *) This->display, This->window, guicursor);
	attrs.cursor = guicursor;
	XChangeWindowAttributes((Display *) This->display, This->window,
				CWCursor, &attrs);
    }
    font_height = font->max_bounds.ascent + font->max_bounds.descent;
    id_width = XTextWidth(font, id, strlen(id));
    if (FullRedraw || This->height <= 80) {
	XFillRectangle((Display *) This->display, This->window, white_gc,
		       0, 0, This->width, This->height);
    } else {
	XFillRectangle((Display *) This->display, This->window, white_gc,
		       BORDER, LOGO_HEIGHT, This->width - BORDER * 2,
		       This->height - font_height - LOGO_HEIGHT - BORDER);
    }
    if (This->height > 80 && This->width > id_width + BORDER * 2) {
	top = BORDER * 2 + LOGO_HEIGHT;
	loops = 0;
	if (FullRedraw) {
	    DrawLogo((Display *) This->display, This->window, black_gc);
	    XDrawString((Display *) This->display, This->window, black_gc,
			This->width - id_width - BORDER,
			This->height - font_height, id, strlen(id));
	}
	// Loop a max of 10 times in case of problems (like too small of a width)
	while (strlen(message) != 0 && loops < 10) {
	    chop = 0;
	    term = 0;
	    while (XTextWidth(font, message, strlen(message) - chop) +
		   BORDER * 2 > This->width)
		chop++;
	    if (strchr(message, '\n') != NULL &&
		strchr(message, '\n') - message < strlen(message) - chop) {
		chop = strlen(message) - (strchr(message, '\n') - message);
		term = 1;
	    }
	    XDrawString((Display *) This->display, This->window, black_gc,
			BORDER, top, message, strlen(message) - chop);
	    message += strlen(message) - chop + term;
	    top += font_height;
	    loops++;
	    if (top > This->height - font_height - BORDER)
		break;
	}
	// Make sure we have at least 100 px in addition to the id tag and borders
	if (percent != -1 && This->width > id_width + BORDER * 3 + 100)
	    DrawProgress((Display *) This->display, This->window, BORDER,
			 This->height - font_height - BORDER,
			 This->width - id_width - BORDER * 3, font_height,
			 percent);

    } else {
	XDrawString((Display *) This->display, This->window, black_gc,
		    BORDER, 15, message, strlen(message));
    }

    XFreeGC((Display *) This->display, black_gc);
    XFreeGC((Display *) This->display, white_gc);

    XFlush((Display *) This->display);
    usleep(100);
}
